package com.chinapnr.mis.lib.ble;

import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothProfile;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.os.Build;
import android.os.ParcelUuid;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresPermission;
import androidx.arch.core.executor.ArchTaskExecutor;

import com.chinapnr.mis.lib.bean.Device;
import com.chinapnr.mis.lib.listener.OnConnectListener;
import com.chinapnr.mis.lib.listener.OnReceiveListener;
import com.chinapnr.mis.lib.listener.OnSelectDeviceListener;
import com.chinapnr.mis.lib.listener.OnSendListener;
import com.chinapnr.mis.lib.listener.OnWriteListener;
import com.chinapnr.mis.lib.ui.CommonListDialog;
import com.chinapnr.mis.lib.ui.WaitingProgressView;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.Vector;

import static android.Manifest.permission.ACCESS_COARSE_LOCATION;
import static android.Manifest.permission.ACCESS_FINE_LOCATION;
import static android.Manifest.permission.ACCESS_NETWORK_STATE;
import static android.Manifest.permission.BLUETOOTH;
import static android.Manifest.permission.BLUETOOTH_ADMIN;
import static android.Manifest.permission.CHANGE_NETWORK_STATE;
import static android.Manifest.permission.INTERNET;

public class BleManager {

    /**
     * 服务UUID
     */
    public static final UUID SERVER_UUID = UUID.fromString("F14581B6-7938-45BA-A8E8-4CF5F6DEB648");
    public static final ParcelUuid SERVER_PARCE_UUID = ParcelUuid.fromString("F14581B6-7938-45BA-A8E8-4CF5F6DEB648");
    /**
     * 写特征UUID
     */
    public static final UUID MIS_RECEIVE_UUID = UUID.fromString("49535343-1E4D-4BD9-BA61-23C647249616");
    /**
     * 读特征UUID
     */
    public static final UUID MIS_SEND_UUID = UUID.fromString("49535343-8841-43F4-A8D4-ECBE34729BB3");

    private static final String TAG = BleManager.class.getSimpleName();
    private OnConnectListener mOnConnectListener;
    private OnReceiveListener onReceiveListener;
    private OnWriteListener onWriteListener;
    private BluetoothAdapter mBluetoothAdapter;
    private static volatile BleManager bleManager;
    private Context mContext;
    private BluetoothLeScanner mBLEScanner;
    private ScanCallback mScanCallback;
    private BluetoothAdapter.LeScanCallback mLeScanCallback ;
    private BluetoothGatt mBluetoothGatt;

    private BluetoothGattCharacteristic writeCharacter;
    private BluetoothGattCharacteristic readCharacter;

    private Vector<byte[]> writeQueue = new Vector<>();
    private byte[] currentSendingByte = null;
    private int offset = 0;
    private int mtu = 20;
    private Vector<Byte> receivedData = new Vector<>();
    private Map<String, String> param = new HashMap<>();

    @RequiresPermission(allOf = {INTERNET, CHANGE_NETWORK_STATE, ACCESS_NETWORK_STATE})
    private BluetoothGattCallback mGattCallback = new BluetoothGattCallback() {

        /**
         * 返回连接状态
         * @param gatt
         * @param status
         * @param newState
         */
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            super.onConnectionStateChange(gatt, status, newState);
            Log.e(TAG,"onConnectionStateChange:[" + gatt.toString() + ","+ status + "," + newState + "]");
            if (status == BluetoothGatt.GATT_SUCCESS){
                switch (newState){
                    case BluetoothProfile.STATE_DISCONNECTED:
                        mOnConnectListener.onError(-1,"连接断开");
                        break;
                    case BluetoothProfile.STATE_CONNECTING:
                        break;
                    case BluetoothProfile.STATE_CONNECTED:
                        startBleServiceSearch();
                        break;
                    case BluetoothProfile.STATE_DISCONNECTING:
                        break;
                }
            }else if (status == 133){
                mBluetoothGatt.close();
                mOnConnectListener.onError(-1,"连接失败,请重新扫描连接设备");
            }else {
                mOnConnectListener.onError(-1,"连接失败,请重试");
            }
        }

        /**
         * 获取到链接设备的GATT服务时的回调
         * @param gatt
         * @param status
         */
        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            super.onServicesDiscovered(gatt, status);
            if (status != BluetoothGatt.GATT_SUCCESS){
                mOnConnectListener.onError(-2,"服务搜索失败，请重试");
            }
            Log.e(TAG,"onServicesDiscovered:[" + gatt.toString() + ","+ status  + "]");
            Log.e(TAG,"onServicesDiscovered:[" + mBluetoothGatt.getServices() + "]");
            List<BluetoothGattService> services = mBluetoothGatt.getServices();

            for (BluetoothGattService gattService :services) {
                List<BluetoothGattCharacteristic> gattCharacteristics =gattService.getCharacteristics();
                Log.e(TAG,"1:BluetoothGattService UUID=:"+gattService.getUuid());
                if (SERVER_UUID.equals(gattService.getUuid())){
                    for (BluetoothGattCharacteristic gattCharacteristic: gattCharacteristics){
                        if (MIS_RECEIVE_UUID.equals(gattCharacteristic.getUuid())){
                            Log.e(TAG,"BluetoothGattCharacteristic 写UUID=:"+gattCharacteristic.getUuid());
                            writeCharacter = gattCharacteristic;
                        }
                        if (MIS_SEND_UUID.equals(gattCharacteristic.getUuid())){
                            readCharacter = gattCharacteristic;
                            Log.e(TAG,"BluetoothGattCharacteristic 读UUID=:"+gattCharacteristic.getUuid());
                        }
                    }
                    writeQueue.clear();
                    if (readCharacter != null && writeCharacter != null){
                        mOnConnectListener.onConnectSuccess(gatt.getDevice().getAddress());
                    }else {
                        mOnConnectListener.onError(-2,"服务搜索失败，请重试");
                    }

                }
            }
        }


        /**
         * 指定特征写入操作的回调结果
         * @param gatt
         * @param characteristic
         * @param status
         */
        @SuppressLint("MissingPermission")
        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            super.onCharacteristicWrite(gatt, characteristic, status);
            mBluetoothGatt.setCharacteristicNotification(writeCharacter, true);
            Log.e(TAG,"onCharacteristicWrite:[" + gatt.getDevice().getName() + "," + characteristic.getUuid() + "," + status  + "]");
            if (currentSendingByte != null){
                Log.e(TAG, "发送数据 Size:" + currentSendingByte.length + " OFFSET[" + (offset - mtu) + "-" + offset + "]");
                if (currentSendingByte.length > offset){
                    writeToBle(currentSendingByte, offset);
                }else{
                    byte[] data = new byte[currentSendingByte.length - 5];
                    System.arraycopy(currentSendingByte, 3, data, 0, currentSendingByte.length - 5);
                    onWriteListener.writeSuccess();
//                    onConnectAndExChangeListener.onSendSuccess(new String(data));
                    writeQueue.removeElementAt(0);
                    if(writeQueue.size() > 0){
                        currentSendingByte = writeQueue.elementAt(0);
                        offset = 0;
                        writeToBle(currentSendingByte, offset);
                    }
                }
            }
        }

        /**
         * 设备发出通知时会回调到该接口
         * @param gatt
         * @param characteristic
         */
        @SuppressLint("MissingPermission")
        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);
            Log.d(TAG,"onCharacteristicChanged:[" + gatt.getDevice().getName() + "," + characteristic.getUuid() + "," + new String(characteristic.getValue()) + "]");
            Log.d(TAG,"onCharacteristicChanged:[" + characteristic.getValue() + "]");
            if (characteristic.getValue() != null){
                onReceived(characteristic.getValue());
            }
        }

        @Override
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            super.onMtuChanged(gatt, mtu, status);
            Log.d(TAG,"onMtuChanged:[" + gatt.toString() + "," + mtu + "," + status  + "]");
        }
    };


    /**
     * Obtains the BleBlueManager getInstance the given context.
     * @param context context
     * @return an instance of BleBlueManager
     */
    public static BleManager getInstance(Context context) {
        if (bleManager == null) {
            synchronized (BleManager.class) {
                if (bleManager == null) {
                    bleManager = new BleManager(context);
                }
            }
        }
        return bleManager;
    }


    /**
     * private constructor for singleton
     * @param context context
     */
    private BleManager(Context context) {
        mContext = context;
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    }


    /**
     * Request for enable the device's bluetooth asynchronously.
     * Throw a NullPointerException if the device doesn't have a bluetooth module.
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void requestEnableBt() {
        if (mBluetoothAdapter == null) {
            throw new NullPointerException("设备没有蓝牙模块");
        }
        if (!mBluetoothAdapter.isEnabled()) {
            mBluetoothAdapter.enable();
        }
    }


    private CommonListDialog commonListDialog;
    /**
     * 搜索并选择设备
     *
     * @param listener 选择设备回调监听
     */
    @RequiresPermission(allOf = {ACCESS_FINE_LOCATION, ACCESS_COARSE_LOCATION, BLUETOOTH, BLUETOOTH_ADMIN})
    public void selectDevice(@NonNull OnSelectDeviceListener listener) {
        commonListDialog = new CommonListDialog(mContext)
                .setTitleText("选择目标蓝牙设备")
                .setCallBackListener(new CommonListDialog.DialogCallback() {
                    @SuppressLint("MissingPermission")
                    @Override
                    public void commit(Device device) {
                        listener.onSuccess(device);
                        stopSearch();
                        release();
                    }

                    @SuppressLint("MissingPermission")
                    @Override
                    public void cancel() {
                        listener.onError(-3, "取消选择");
                        stopSearch();
                        release();
                    }
                });
        commonListDialog.show();
        stopSearch();
        startSearch(null, listener);
    }

    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    private void startSearch(Device defaultDevice, OnSelectDeviceListener listener){
        param.clear();
        if(android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Log.d(TAG, "mBLEScanner startScan");
            //获取5.0新添的扫描类
            startLeScanner(defaultDevice, listener);
        } else {
            //5.0以下  开始扫描
            startLeScan(defaultDevice, listener);
        }
    }


    /**
     * Android 5.0以上搜索BLE蓝牙
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    private void startLeScanner(Device defaultDevice, OnSelectDeviceListener onSelectDeviceListener){
        if (mBLEScanner == null){
            mBLEScanner = mBluetoothAdapter.getBluetoothLeScanner();
        }

        mScanCallback = new ScanCallback() {
            @SuppressLint("MissingPermission")
            @Override
            public void onScanResult(int callbackType, ScanResult result) {
                super.onScanResult(callbackType, result);
                if (result != null && result.getScanRecord() != null && result.getScanRecord().getServiceUuids() != null
                        && result.getScanRecord().getServiceUuids().contains(SERVER_PARCE_UUID)){
                    Log.d(TAG, "callbackType:" + callbackType + ", result[" +  result.toString() + "]");
                    BluetoothDevice device = result.getDevice();
                    if (defaultDevice != null){
                        if (defaultDevice.getName().equals(device.getName())){
                            onSelectDeviceListener.onSuccess(new Device(device.getName(), device.getAddress(), false));
                        }
                    }else {
                        if (param.containsKey(device.getAddress())){
                            return;
                        }
                        if (!TextUtils.isEmpty(device.getName()) && !TextUtils.isEmpty(device.getAddress())
                                && device.getName().startsWith("PNR")){
                            param.put(device.getAddress(), device.getName());
                            if (commonListDialog != null){
                                commonListDialog.addData(new Device(device.getName(), device.getAddress(), false));
                            }
                        }
                    }

                }
            }

            @Override
            public void onBatchScanResults(List<ScanResult> results) {
                super.onBatchScanResults(results);
            }

            @Override
            public void onScanFailed(int errorCode) {
                super.onScanFailed(errorCode);
                onSelectDeviceListener.onError(-2, getScanErrorMessage(errorCode));
            }
        };
        ScanSettings.Builder builder = new ScanSettings.Builder();
        builder.setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY);
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.M){
            builder.setCallbackType(ScanSettings.CALLBACK_TYPE_ALL_MATCHES);
            builder.setMatchMode(ScanSettings.MATCH_MODE_STICKY);
        }
        builder.setReportDelay(0L);
        mBLEScanner.startScan(null, builder.build(),mScanCallback);
    }

    /**
     * Android 5.0以下搜索BLE蓝牙
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    private void startLeScan(Device defaultDevice, OnSelectDeviceListener onSelectDeviceListener){
        mLeScanCallback = (device, rssi, scanRecord) -> {
            Log.e(TAG, "onLeScan, device: " + device.getName() + ", " + device.getAddress() + ", " + device.getUuids());
            if (defaultDevice != null){
                if (defaultDevice.getName().equals(device.getName())){
                    onSelectDeviceListener.onSuccess(new Device(device.getName(), device.getAddress(), false));
                }
            }else {
                if (TextUtils.isEmpty(device.getName()) || TextUtils.isEmpty(device.getAddress()) || device.getName().startsWith("PNR")){
                    return;
                }
                if (!param.containsKey(device.getAddress())){
                    param.put(device.getAddress(), device.getName());
                    commonListDialog.addData(new Device(device.getName(), device.getAddress(), false));
                }
            }
        };
        if (mBluetoothAdapter.isDiscovering()){
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
        }
        mBluetoothAdapter.startLeScan(mLeScanCallback);
    }

    /**
     * 停止搜索蓝牙
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void stopSearch(){
        if(android.os.Build.VERSION.SDK_INT >= 21) {
            if (mBLEScanner != null){
                mBLEScanner.stopScan(mScanCallback);
            }
            mScanCallback = null;
        } else {
            if (mBluetoothAdapter != null){
                mBluetoothAdapter.stopLeScan(mLeScanCallback);
            }
            mLeScanCallback = null;
        }
        mBLEScanner = null;

    }


    /**
     * 搜索蓝牙服务
     */
    public void startBleServiceSearch(){
        mBluetoothGatt.discoverServices();
    }


    private void initConnectData(){
        Log.e(TAG, "初始化发送参数配置");
        if (mBluetoothGatt != null && mBluetoothGatt.connect()){
            mBluetoothGatt.disconnect();
            mBluetoothGatt.close();
        }
        readCharacter = null;
        writeCharacter = null;
        if (receivedData ==null){
            receivedData = new Vector<>();
        }
        receivedData.clear();
        if (writeQueue == null){
            writeQueue = new Vector<>();
        }
        writeQueue.clear();
        currentSendingByte = null;
        offset = 0;
        mtu = 20;

    }

    /**
     * 连接指定设备
     * 适用于先选择设备并用选择的设备进行连接
     * @param macAddress
     * @param onConnectListener
     */
    @SuppressLint("RestrictedApi")
    public void connectDevice(String macAddress, OnConnectListener onConnectListener){
        try {
            this.mOnConnectListener = onConnectListener;
            initConnectData();
            if (macAddress == null || TextUtils.isEmpty(macAddress)) {
                throw new IllegalArgumentException("mac address is null or empty!");
            }
            if (!BluetoothAdapter.checkBluetoothAddress(macAddress)) {
                throw new IllegalArgumentException("mac address is not correct! make sure it's upper case!");
            }

            BluetoothDevice mBluetoothDevice = mBluetoothAdapter.getRemoteDevice(macAddress);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                mBluetoothGatt = mBluetoothDevice.connectGatt(mContext, false, mGattCallback, BluetoothDevice.TRANSPORT_LE);
            } else {
                mBluetoothGatt = mBluetoothDevice.connectGatt(mContext, false, mGattCallback);
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        }
    }

    /**
     * 连接指定设备
     * 适用于目标设备信息为历史保存的设备
     * @param defaultDevice
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void connectDevice(Device defaultDevice, OnConnectListener onConnectListener) {
        try {
            this.mOnConnectListener = onConnectListener;

            startSearch(defaultDevice, new OnSelectDeviceListener() {
                @SuppressLint("MissingPermission")
                @Override
                public void onSuccess(Device device) {
                    stopSearch();
                    initConnectData();
                    if (!BluetoothAdapter.checkBluetoothAddress(device.getAddress())) {
                        throw new IllegalArgumentException("mac address is not correct! make sure it's upper case!");
                    }
                    BluetoothDevice mBluetoothDevice = mBluetoothAdapter.getRemoteDevice(device.getAddress());
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                        mBluetoothGatt = mBluetoothDevice.connectGatt(mContext, false, mGattCallback, BluetoothDevice.TRANSPORT_LE);
                    } else {
                        mBluetoothGatt = mBluetoothDevice.connectGatt(mContext, false, mGattCallback);
                    }
                }

                @Override
                public void onError(int code, String message) {
                    onConnectListener.onError(code, message);
                }
            });


        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        }
    }

    /**
     * 断开蓝牙，回收内存
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void closeBleManager(){
        stopSearch();
        closeConnection();
        release();
    }

    /**
     * 彻底关闭连接，不带onConnectionStateChange回调的
     */
    public void closeConnection() {
        if (mBluetoothGatt != null){
            mBluetoothGatt.disconnect();
            mBluetoothGatt.close();
        }

    }

    /**
     * 释放内存
     */
    public void release(){
        mBluetoothAdapter = null;
        mBLEScanner = null;
        mBluetoothGatt = null;
        writeCharacter = null;
        readCharacter = null;
        bleManager = null;
        mContext = null;
    }

    /**
     * 向指定设备发送消息
     * 适用于先选择设备并用选择的设备进行连接
     * @param macAddress
     * @param message
     * @param listener
     */
    @SuppressLint("RestrictedApi")
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void sendMessage(String macAddress, String message, boolean showDialog, OnSendListener listener) {
        Log.d(TAG, "向连接设备[" + macAddress + "]发送数据");
        OnWriteListener writeListener = new OnWriteListener() {
            @Override
            public void writeSuccess() {
                Log.d(TAG, "发送成功");
                if (showDialog) {
//                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("交易中，请稍后...", ""));
                }
                listener.onSendSuccess();
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                listener.onError(code, message);
            }
        };
        OnReceiveListener receiveListener = new OnReceiveListener() {
            @SuppressLint("MissingPermission")
            @Override
            public void onReceive(String data) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                listener.onReceiveSuccess(data);
                closeBleManager();
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                listener.onError(code, message);
            }
        };
        if (showDialog) {
            ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("连接中，请稍后...", ""));
        }
        connectDevice(macAddress, new OnConnectListener() {
            @Override
            public void onConnectSuccess(String mac) {
                if (showDialog) {
//                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("发送中，请稍后...", ""));
                }
                writeData(message, writeListener, receiveListener);
            }

            @Override
            public void onError(int code, String message) {
                listener.onError(code, message);
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
            }
        });
    }


    /**
     * 向指定设备发送消息
     * 适用于目标设备信息为历史保存的设备
     * @param device
     */
    @SuppressLint("RestrictedApi")
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void sendMessage(Device device, String message, boolean showDialog, OnSendListener listener) {
        Log.d(TAG, "向历史设备[" + device.toString() + "]发送数据");
        OnWriteListener writeListener = new OnWriteListener() {
            @Override
            public void writeSuccess() {
                Log.d(TAG, "发送成功");
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("交易中，请稍后...", ""));
                }
                listener.onSendSuccess();
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                listener.onError(code, message);
            }
        };
        OnReceiveListener receiveListener = new OnReceiveListener() {
            @SuppressLint("MissingPermission")
            @Override
            public void onReceive(String data) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                listener.onReceiveSuccess(data);
                closeBleManager();
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                listener.onError(code, message);
            }
        };
        if (showDialog) {
            ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("连接中，请稍后...", ""));
        }
        connectDevice(device, new OnConnectListener() {
            @Override
            public void onConnectSuccess(String mac) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("发送中，请稍后...", ""));
                }
                writeData(message, writeListener, receiveListener);
            }

            @Override
            public void onError(int code, String message) {
                listener.onError(code, message);
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
            }
        });
    }

    private boolean writeData(String message, OnWriteListener onWriteListener, OnReceiveListener onReceiveListener){
        this.onWriteListener = onWriteListener;
        this.onReceiveListener = onReceiveListener;
        if (mBluetoothGatt == null){
            return false;
        }
        mBluetoothGatt.setCharacteristicNotification(writeCharacter, true);
        if(readCharacter != null) {
            byte[] packData = pack(message.getBytes());
            writeQueue.addElement(packData);
            if(writeQueue.size() == 1) {
                int offset = 0;
                currentSendingByte = packData;
                writeToBle(currentSendingByte, offset);
            }
            return true;
        }else{
            return false;
        }
    }

    private void writeToBle(byte[] bytes, int offset) {
        if (bytes == null){
            return;
        }
        if (bytes.length - offset < mtu) {
            if (readCharacter != null) {
                byte[] data = new byte[bytes.length - offset];
                System.arraycopy(bytes, offset, data, 0, bytes.length - offset );
                readCharacter.setValue(data);
            }
        } else {
            if (readCharacter != null) {
                byte[] data = new byte[mtu];
                System.arraycopy(bytes, offset, data, 0, offset + mtu - offset );
                readCharacter.setValue(data);
            }
        }
        this.offset += mtu;
        mBluetoothGatt.writeCharacteristic(readCharacter);
    }

    /**
     * 打包
     */
    private byte[] pack(byte[] data){
        byte begin = 0x2;
        byte end = 0x3;
        int length = data.length;
        byte lenH = (byte)((length & '\uff00') >> 8);
        byte lenL = (byte)(length & 255);
        byte[] dt = new byte[length + 5];
        dt[0] = begin;
        dt[1] = lenH;
        dt[2] = lenL;
        for(int i = 0; i < data.length; ++i) {
            dt[i + 3] = data[i];
        }
        dt[data.length + 3] = end;
        dt[data.length + 4] = caculateLRC(data);
        return dt;
    }

    /**
     * 计算LRC
     */
    private byte caculateLRC(byte[] bytes){
        byte headLrcValue = 0x0;
        for (byte aBuffer : bytes) {
            headLrcValue ^= aBuffer;
        }
        return headLrcValue;
    }

    /**
     * 接收数据并校验数据完整性
     * @param data
     */
    private void onReceived(byte[] data){
        if(receivedData.size() == 0){
            if(data[0] != 0x2){
                return;
            }
        }
        for(byte bt : data){
            receivedData.add(bt);
        }
        Log.d(TAG, "收到:" + receivedData.size());
        if(receivedData.size() > 3){
            int size =  receivedData.get(1) * 256  + receivedData.get(2);
            if(receivedData.size() >= size + 5){
                List<Byte> bytes = receivedData.subList(0, size + 5);
                List<Byte> dt = bytes.subList(3, size + 3);
                byte[] byteArray = new byte[dt.size()];
                for (int i = 0; i < dt.size(); i++){
                    byteArray[i] = dt.get(i);
                }
                byte lrc = caculateLRC(byteArray);
                if(lrc == bytes.get(bytes.size() - 1 ) && bytes.get(size + 3) == 0x3) {
                    onReceiveListener.onReceive(new String(byteArray));
                }
                List<Byte> t = receivedData.subList(size + 5, receivedData.size());
                Vector<Byte> tt = new Vector<>(t.size());
                for(Byte b : t){
                    tt.add(b);
                }
                receivedData.clear();
                receivedData.addAll(tt);
            }
        }
    }

    private String getScanErrorMessage(int errorCode){
        String message;
        switch (errorCode) {
            case ScanCallback.SCAN_FAILED_ALREADY_STARTED:
                message = "已经开启扫描";
                break;
            case ScanCallback.SCAN_FAILED_APPLICATION_REGISTRATION_FAILED:
                message = "扫描太频繁";
                break;
            case ScanCallback.SCAN_FAILED_INTERNAL_ERROR:
                message = "内部错误";
                break;
            case ScanCallback.SCAN_FAILED_FEATURE_UNSUPPORTED:
                message = "设备不支持";
                break;
            default:
                message = "扫描异常";
                break;
        }
        return message;
    }


}
