package com.chinapnr.mis.lib.net;


import android.annotation.SuppressLint;
import android.content.Context;
import android.util.Log;

import androidx.annotation.RequiresPermission;
import androidx.arch.core.executor.ArchTaskExecutor;

import com.chinapnr.mis.lib.bean.MessageData;
import com.chinapnr.mis.lib.listener.OnConnectListener;
import com.chinapnr.mis.lib.listener.OnReceiveListener;
import com.chinapnr.mis.lib.listener.OnSendListener;
import com.chinapnr.mis.lib.listener.OnWriteListener;
import com.chinapnr.mis.lib.ui.WaitingProgressView;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import static android.Manifest.permission.ACCESS_NETWORK_STATE;
import static android.Manifest.permission.CHANGE_NETWORK_STATE;
import static android.Manifest.permission.INTERNET;


public class NetManager {
    private static final String TAG = NetManager.class.getSimpleName();
    private static NetManager netManager;
    private Context mContext;

    private ExecutorService executor = Executors.newCachedThreadPool();
    private Future future;
    private Socket client;
    private InputStream inputStream;
    private OutputStream outputStream;

    private NetManager(Context mContext) {
        this.mContext = mContext;
    }

    public static NetManager getInstance(Context context){
        if (netManager == null){
            synchronized (NetManager.class){
                if (netManager == null){
                    netManager = new NetManager(context);
                }
            }
        }
        return netManager;
    }
    @SuppressLint("RestrictedApi")
    @RequiresPermission(allOf = {INTERNET, CHANGE_NETWORK_STATE, ACCESS_NETWORK_STATE})
    public void sendMessage(String ipAddress, String message, boolean showDialog, OnSendListener onSendListener){
        OnReceiveListener onReceiveListener = new OnReceiveListener() {

            @Override
            public void onReceive(String data) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onReceiveSuccess(data);
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onError(code, message);
            }
        };

        OnWriteListener onWriteListener = new OnWriteListener() {
            @Override
            public void writeSuccess() {
                if (showDialog) {
//                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("交易中，请稍后...", ""));
                }
                onSendListener.onSendSuccess();
                readData(onReceiveListener);
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onError(code, message);
            }
        };
        OnConnectListener onConnectListener = new OnConnectListener() {
            @Override
            public void onConnectSuccess(String mac) {
                if (showDialog) {
//                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("发送中，请稍后...", ""));
                }
                writeData(new MessageData(message), onWriteListener);

            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onError(code, message);
            }
        };
        if (showDialog) {
            ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("连接中，请稍后...", ""));
        }
        connect(ipAddress, onConnectListener);
    }

    @RequiresPermission(allOf = {INTERNET, CHANGE_NETWORK_STATE, ACCESS_NETWORK_STATE})
    private void connect(String ipAddress, OnConnectListener onConnectListener){
        future = executor.submit(() -> {
            try {
                client = new Socket(ipAddress, 6000);

//                client.connect();
                inputStream = client.getInputStream();
                outputStream = client.getOutputStream();
                onConnectListener.onConnectSuccess("");
            } catch (Exception e) {
                e.printStackTrace();
                onConnectListener.onError(-1, "连接失败");
            }
        });

    }

    /**
     * 发送消息
     * @param data 消息内容
     * @param onWriteListener 发送监听
     */
    @SuppressLint("RestrictedApi")
    public void writeData(MessageData data, OnWriteListener onWriteListener) {

        future = executor.submit(() -> {
            try {
                outputStream.flush();
                outputStream.write(data.text.getBytes());
                onWriteListener.writeSuccess();
            } catch (IOException e) {
                e.printStackTrace();
                onWriteListener.onError(-5, "发送数据失败：" + e.toString());
            }

        });

    }

    /**
     * 接收消息
     * @param onReceiveListener 接收监听
     */
    @SuppressLint("RestrictedApi")
    public void readData(OnReceiveListener onReceiveListener) {
        future = executor.submit(() -> {
            try {
                int count = 0;
                while (count == 0) {
                    count = inputStream.available();
                    if (!client.isConnected()){
                        onReceiveListener.onError(-6, "连接断开");
                        close();
                        return;
                    }
                }
                Log.d(TAG, "数据字节长度：" + count);
                byte[] buffer = new byte[count];
                int length = inputStream.read(buffer);
                Log.e(TAG, "接收数据的长度：" + length);
                if (length > 0) {
                    onReceiveListener.onReceive(new String(buffer));
                } else if (length == 0) {
                    onReceiveListener.onReceive("");
                } else {
                    onReceiveListener.onError(-6, "接收数据错误");
                }
                close();
            } catch (IOException e) {
                e.printStackTrace();
                onReceiveListener.onError(-7, "接收数据异常：" + e.getMessage());
            }
        });
    }

    public void close() {
        try {
            if (inputStream != null) {
                inputStream.close();
            }
            inputStream = null;

            if (outputStream != null) {
                outputStream.close();
            }
            outputStream = null;


            if (executor != null) {
                executor.shutdown();
            }
            if (future != null) {
                future.cancel(true);
            }

            executor = null;
            future = null;

            mContext = null;
            netManager = null;


        } catch (IOException e) {
            e.printStackTrace();
        }

    }


}
