package com.chinapnr.mis.lib.socket;

import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresPermission;
import androidx.arch.core.executor.ArchTaskExecutor;

import com.chinapnr.mis.lib.bean.Device;
import com.chinapnr.mis.lib.bean.MessageData;
import com.chinapnr.mis.lib.config.Constants;
import com.chinapnr.mis.lib.listener.OnConnectListener;
import com.chinapnr.mis.lib.listener.OnReceiveListener;
import com.chinapnr.mis.lib.listener.OnSelectDeviceListener;
import com.chinapnr.mis.lib.listener.OnSendListener;
import com.chinapnr.mis.lib.listener.OnWriteListener;
import com.chinapnr.mis.lib.ui.CommonListDialog;
import com.chinapnr.mis.lib.ui.WaitingProgressView;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import static android.Manifest.permission.BLUETOOTH;
import static android.Manifest.permission.BLUETOOTH_ADMIN;

public class SocketManager {

    private static final String TAG = SocketManager.class.getSimpleName();

    private static SocketManager socketManager;
    private Context mContext;

    private BluetoothAdapter bluetoothAdapter;

    private List<Device> deviceList = new ArrayList<>();
    private CommonListDialog commonListDialog;
    private OnSelectDeviceListener selectDeviceListener;

    private ExecutorService executor = Executors.newCachedThreadPool();
    private Future future;

    private SocketManager(Context context) {
        this.mContext = context;
        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    }

    public static SocketManager getInstance(Context context) {
        if (socketManager == null) {
            synchronized (SocketManager.class) {
                if (socketManager == null) {
                    socketManager = new SocketManager(context);
                }
            }
        }
        return socketManager;
    }

    /**
     * 搜索并选择设备
     *
     * @param listener 选择设备回调监听
     */
    @RequiresPermission(allOf = { BLUETOOTH, BLUETOOTH_ADMIN})
    public void selectDevice(@NonNull OnSelectDeviceListener listener) {
        this.selectDeviceListener = listener;
        commonListDialog = new CommonListDialog(mContext)
                .setTitleText("选择目标蓝牙设备")
                .setCallBackListener(new CommonListDialog.DialogCallback() {
                    @SuppressLint("MissingPermission")
                    @Override
                    public void commit(Device device) {
                        stopSearch();
                        Log.d(TAG, "选中了：" + device.getName());
                        if (device.isBond()) {
                            selectDeviceListener.onSuccess(device);
                            stopSelect();
                        } else {
                            createBond(device);
                        }
                    }

                    @SuppressLint("MissingPermission")
                    @Override
                    public void cancel() {
                        stopSearch();
                        selectDeviceListener.onError(-3, "取消选择");
                        stopSelect();
                    }
                });

        searchDevice();


    }

    /**
     * 搜索设备
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    private void searchDevice() {
        try {
            if (deviceList == null) {
                deviceList = new ArrayList<>();
            }
            deviceList.clear();

            IntentFilter filter = new IntentFilter();
            //蓝牙状态改变的广播
            filter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
            //找到设备的广播
            filter.addAction(BluetoothDevice.ACTION_FOUND);
            //搜索完成的广播
            filter.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
            //开始扫描的广播
            filter.addAction(BluetoothAdapter.ACTION_DISCOVERY_STARTED);
            //状态改变
            filter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);

            mContext.registerReceiver(searchReceiver, filter);

            Set<BluetoothDevice> bluetoothDevices = bluetoothAdapter.getBondedDevices();
            for (BluetoothDevice bluetoothDevice : bluetoothDevices) {
                Log.e(TAG, "已经配对的蓝牙设备：" + bluetoothDevice.getName() + "," + bluetoothDevice.getAddress());
                if (bluetoothDevice.getName().startsWith("PNR")) {
                    Device searchResult = new Device(bluetoothDevice.getName(), bluetoothDevice.getAddress());
                    deviceList.add(searchResult);
                }
            }
            if (commonListDialog != null) {
                commonListDialog.setListData(deviceList).show();
            }
            if (!bluetoothAdapter.isDiscovering()) {
                bluetoothAdapter.startDiscovery();
            }
        } catch (Exception e) {
            e.printStackTrace();
            selectDeviceListener.onError(-11, "搜索设备异常：" + e.getMessage());
            stopSelect();
        }
    }

    /**
     * 停止搜索设备
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    private void stopSearch() {
        try {
            mContext.unregisterReceiver(searchReceiver);
            if (bluetoothAdapter.isDiscovering()) {
                bluetoothAdapter.cancelDiscovery();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 绑定设备
     * @param device
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void createBond(Device device) {
        try {
            IntentFilter filter = new IntentFilter();
            //蓝牙状态改变的广播
            filter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
            //状态改变
            filter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);

            mContext.registerReceiver(bondReceiver, filter);
            BluetoothDevice bluetoothDevice = bluetoothAdapter.getRemoteDevice(device.getAddress());

            boolean flag = bluetoothDevice.createBond();
            // 反射调用配对设备，某些android版本需要通过反射来调用
            /*  Method method = BluetoothDevice.class.getMethod("createBond");   */
            /*  boolean flag = (boolean) method.invoke(device);   */
            WaitingProgressView.getInstance(mContext).show("配对中，请稍后...", "");

            Log.e(TAG, "开始配对：" + flag);
        } catch (Exception e) {
            e.printStackTrace();
            selectDeviceListener.onError(-12, "配对设备异常：" + e.getMessage());
        }
    }

    /**
     * 关闭设备绑定
     */
    private void closeBond() {
        try {
            mContext.unregisterReceiver(bondReceiver);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 停止搜索设备
     */
    private void stopSelect() {
        bluetoothAdapter = null;
        mContext = null;
        socketManager = null;
    }

    /**
     * 搜索设备广播监听
     */
    @RequiresPermission(allOf = { BLUETOOTH, BLUETOOTH_ADMIN})
    private BroadcastReceiver searchReceiver = new BroadcastReceiver() {
        @SuppressLint("MissingPermission")
        @Override
        public void onReceive(Context context, Intent intent) {
            try {
                String action = intent.getAction();
                switch (action) {
                    case BluetoothAdapter.ACTION_DISCOVERY_STARTED:
                        if (commonListDialog != null) {
                            commonListDialog.showProgress();
                        }
                        break;
                    case BluetoothAdapter.ACTION_DISCOVERY_FINISHED:
                        if (commonListDialog != null) {
                            commonListDialog.hideProgress();
                        }
                        if (deviceList == null) {
                            if (commonListDialog != null) {
                                commonListDialog.dismiss();
                            }
                            if (selectDeviceListener != null) {
                                selectDeviceListener.onError(-1, "未搜索到设备，请检查POS机状态");
                            }
                        }
                        break;
                    case BluetoothDevice.ACTION_FOUND:
                        BluetoothDevice bluetoothDevice = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                        String deviceStr = "found device["
                                + "name:" + bluetoothDevice.getName() + ","
                                + "address:" + bluetoothDevice.getAddress() + ","
                                + "bondState:" + bluetoothDevice.getBondState() + ","
                                + "type:" + bluetoothDevice.getType() + ","
                                + "bluetoothClass:" + bluetoothDevice.getBluetoothClass() + ","
                                + "uuids:" + bluetoothDevice.getUuids()
                                + "]";
                        Log.e(TAG, deviceStr);
                        for (Device device : deviceList) {
                            if (device.getName().equals(bluetoothDevice.getName()) && device.getAddress().equals(bluetoothDevice.getAddress())) {
                                device.setBond(bluetoothDevice.getBondState() == BluetoothDevice.BOND_BONDED);
                                return;
                            }
                        }
                        if (!TextUtils.isEmpty(bluetoothDevice.getName()) && bluetoothDevice.getName().startsWith("PNR")
                                && BluetoothDevice.DEVICE_TYPE_CLASSIC == bluetoothDevice.getType()) {
                            Device device = new Device(bluetoothDevice.getName(), bluetoothDevice.getAddress());
                            device.setBond(bluetoothDevice.getBondState() == BluetoothDevice.BOND_BONDED);
                            if (commonListDialog != null) {
                                deviceList.add(device);
                                commonListDialog.updateData();
                                if (!commonListDialog.isShowing()) {
                                    commonListDialog.show();
                                }
                            }
                        }
                        break;

                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * 配对设备广播监听
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    private BroadcastReceiver bondReceiver = new BroadcastReceiver() {
        @SuppressLint("MissingPermission")
        @Override
        public void onReceive(Context context, Intent intent) {
            try {
                String action = intent.getAction();
                if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
                    BluetoothDevice bondDevice = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                    String bondDeviceStr = "bond device["
                            + "name:" + bondDevice.getName() + ","
                            + "address:" + bondDevice.getAddress() + ","
                            + "bondState:" + bondDevice.getBondState() + ","
                            + "type:" + bondDevice.getType() + ","
                            + "bluetoothClass:" + bondDevice.getBluetoothClass() + ","
                            + "uuids:" + bondDevice.getUuids()
                            + "]";
                    Log.e(TAG, bondDeviceStr);

                    switch (bondDevice.getBondState()) {
                        case BluetoothDevice.BOND_BONDING:
                            //正在配对
                            Log.d(TAG, "正在配对");
                            break;
                        case BluetoothDevice.BOND_BONDED:
                            //配对结束
                            closeBond();
                            WaitingProgressView.getInstance(mContext).dismiss();
                            Device device = new Device(bondDevice.getName(), bondDevice.getAddress());
                            selectDeviceListener.onSuccess(device);
                            stopSelect();
                            Log.d(TAG, "配对成功");
                            break;
                        case BluetoothDevice.BOND_NONE:
                            //取消配对/未配对
                            closeBond();
                            WaitingProgressView.getInstance(mContext).dismiss();
                            selectDeviceListener.onError(-2, "配对失败");
                            stopSelect();
                            Log.d(TAG, "取消配对/未配对");
                        default:
                            break;
                    }
                }

            } catch (Exception e) {
                Log.e(TAG, "监听配对状态异常：" + e.toString());
                e.printStackTrace();
            }
        }
    };

    private BluetoothSocket bluetoothSocket;
    private InputStream inputStream;
    private OutputStream outputStream;

    /**
     * 向指定设备发送消息
     * @param device 设备信息
     * @param message 消息内容
     * @param onSendListener 回调监听
     */
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void sendMessage(Device device, String message, OnSendListener onSendListener) {
        sendMessage(device, message,false, onSendListener);
    }
    /**
     * 向指定设备发送消息
     * @param device 设备信息
     * @param message 消息内容
     * @param showDialog 是否显示等待对话框
     * @param onSendListener 回调监听
     */
    @SuppressLint("RestrictedApi")
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void sendMessage(Device device, String message, boolean showDialog, OnSendListener onSendListener) {
        OnReceiveListener onReceiveListener = new OnReceiveListener() {
            @Override
            public void onReceive(String data) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onReceiveSuccess(data);
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onError(code, message);
            }
        };

        OnWriteListener onWriteListener = new OnWriteListener() {
            @Override
            public void writeSuccess() {
                if (showDialog) {
//                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("交易中，请稍后...", ""));
                }
                readData(onReceiveListener);
                onSendListener.onSendSuccess();
            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onError(code, message);
            }
        };
        OnConnectListener onConnectListener = new OnConnectListener() {
            @Override
            public void onConnectSuccess(String mac) {
                if (showDialog) {
//                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("发送中，请稍后...", ""));
                }
                writeData(new MessageData(message), onWriteListener);

            }

            @Override
            public void onError(int code, String message) {
                if (showDialog) {
                    ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).dismiss());
                }
                onSendListener.onError(code, message);
            }
        };
        if (showDialog) {
            ArchTaskExecutor.getMainThreadExecutor().execute(() -> WaitingProgressView.getInstance(mContext).show("连接中，请稍后...", ""));
        }
        connect(device, onConnectListener);
    }

    /**
     * 连接设备
     * @param device 设备信息
     * @param onConnectListener 连接监听
     */
    @SuppressLint("RestrictedApi")
    @RequiresPermission(allOf = {BLUETOOTH, BLUETOOTH_ADMIN})
    public void connect(Device device, OnConnectListener onConnectListener) {

        future = executor.submit(() -> {
            BluetoothDevice remoteDevice = bluetoothAdapter.getRemoteDevice(device.getAddress());
            try {
                bluetoothSocket = remoteDevice.createInsecureRfcommSocketToServiceRecord(UUID.fromString(Constants.STR_UUID));
                bluetoothSocket.connect();

                inputStream = bluetoothSocket.getInputStream();
                outputStream = bluetoothSocket.getOutputStream();

                onConnectListener.onConnectSuccess(device.getAddress());

            } catch (IOException e) {
                e.printStackTrace();
                onConnectListener.onError(-21, "连接失败");
                close();
            }
        });
    }

    /**
     * 发送消息
     * @param data 消息内容
     * @param onWriteListener 发送监听
     */
    @SuppressLint("RestrictedApi")
    public void writeData(MessageData data, OnWriteListener onWriteListener) {

        future = executor.submit(() -> {
            try {
                outputStream.flush();
                outputStream.write(data.text.getBytes());
                onWriteListener.writeSuccess();
            } catch (IOException e) {
                e.printStackTrace();
                onWriteListener.onError(-5, "发送数据失败：" + e.toString());

            }

        });

    }

    /**
     * 接收消息
     * @param onReceiveListener 接收监听
     */
    @SuppressLint("RestrictedApi")
    public void readData(OnReceiveListener onReceiveListener) {
        future = executor.submit(() -> {
            Log.d(TAG, "等待接收数据");
            try {
                int count = 0;
                while (count == 0) {
                    count = inputStream.available();
                }
                Log.d(TAG, "数据字节长度：" + count);
                byte[] buffer = new byte[count];
                int length = inputStream.read(buffer);
                Log.e(TAG, "接收数据的长度：" + length);
                if (length > 0) {
                    onReceiveListener.onReceive(new String(buffer));
                } else if (length == 0) {
                    onReceiveListener.onReceive("");
                } else {
                    onReceiveListener.onError(-6, "接收数据错误");
                }
                close();
            } catch (IOException e) {
                e.printStackTrace();
                onReceiveListener.onError(-7, "接收数据异常：" + e.getMessage());
            }
        });
    }


    public void close() {
        try {
            if (inputStream != null) {
                inputStream.close();
            }
            inputStream = null;

            if (outputStream != null) {
                outputStream.close();
            }
            outputStream = null;

            if (bluetoothSocket != null) {
                bluetoothSocket.close();
            }
            bluetoothSocket = null;

            if (executor != null) {
                executor.shutdown();
            }
            if (future != null) {
                future.cancel(true);
            }

            executor = null;
            future = null;

            bluetoothAdapter = null;
            mContext = null;
            socketManager = null;


        } catch (IOException e) {
            e.printStackTrace();
        }

    }

}
